@@ -86,6 +86,12 @@ extern int dual_allocation;
 
 extern char origin[STUN_MAX_ORIGIN_SIZE+1];
 
+extern int oauth;
+extern oauth_key okey_array[2];
+
+#define UCLIENT_SESSION_LIFETIME (777)
+#define OAUTH_SESSION_LIFETIME (555)
+
 #define is_TCP_relay() (relay_transport == STUN_ATTRIBUTE_TRANSPORT_TCP_VALUE)
 
 void start_mclient(const char *remote_address, int port,
@@ -100,6 +106,7 @@ void client_input_handler(evutil_socket_t fd, short what, void* arg);
 turn_credential_type get_turn_credentials_type(void);
 
 int add_integrity(app_ur_conn_info *clnet_info, stun_buffer *message);
+int check_integrity(app_ur_conn_info *clnet_info, stun_buffer *message);
 
 void recalculate_restapi_hmac(void);
 
