@@ -39,3 +39,16 @@ CREATE TABLE turn_realm_option (
 	value varchar(128),
 	primary key (realm,opt)
 );
+
+CREATE TABLE oauth_key (
+	kid varchar(128),
+	ikm_key varchar(256) default '',
+	timestamp bigint default 0,
+	lifetime integer default 0,
+	hkdf_hash_func varchar(64) default '',
+	as_rs_alg varchar(64) default '',
+	as_rs_key varchar(256) default '',
+	auth_alg varchar(64) default '',
+	auth_key varchar(256) default '',
+	primary key (kid)
+);
